package kr.co.vividnext.sodalive.audition.applicant

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.CircleCropTransformation
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.databinding.ItemAuditionApplicantBinding

class AuditionApplicantListAdapter(
    private val onClickVote: (Int) -> Unit,
    private val onClickPlayOrPause: (Int, Long, String) -> Unit
) : ListAdapter<GetAuditionRoleApplicantItem, AuditionApplicantListAdapter.ViewHolder>(DiffCallback()) {

    private var currentPlayingIndex: Int = -1
    private var isPlaying: Boolean = false

    class DiffCallback : DiffUtil.ItemCallback<GetAuditionRoleApplicantItem>() {
        override fun areItemsTheSame(
            oldItem: GetAuditionRoleApplicantItem,
            newItem: GetAuditionRoleApplicantItem
        ): Boolean {
            return oldItem.applicantId == newItem.applicantId
        }

        override fun areContentsTheSame(
            oldItem: GetAuditionRoleApplicantItem,
            newItem: GetAuditionRoleApplicantItem
        ): Boolean {
            return oldItem == newItem
        }
    }

    inner class ViewHolder(
        private val binding: ItemAuditionApplicantBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        fun bind(item: GetAuditionRoleApplicantItem) {
            binding.llVote.setOnClickListener { onClickVote(bindingAdapterPosition) }
            binding.ivProfile.setOnClickListener {
                onClickPlayOrPause(bindingAdapterPosition, item.applicantId, item.voiceUrl)
            }

            binding.tvNickname.text = item.nickname
            binding.tvCountVote.text = item.voteCount.toString()
            binding.ivProfile.load(item.profileImageUrl) {
                crossfade(true)
                placeholder(R.drawable.bg_placeholder)
                transformations(CircleCropTransformation())
            }

            binding.ivPlayOrPause.setImageResource(
                if (bindingAdapterPosition == currentPlayingIndex && isPlaying) {
                    R.drawable.ic_audition_pause
                } else {
                    R.drawable.ic_audition_play
                }
            )
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int) = ViewHolder(
        ItemAuditionApplicantBinding.inflate(
            LayoutInflater.from(parent.context),
            parent,
            false
        )
    )

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bind(getItem(position))
    }

    fun updatePlayingIndex(newIndex: Int, isPlaying: Boolean) {
        this.isPlaying = isPlaying

        val previousIndex = currentPlayingIndex
        currentPlayingIndex = newIndex
        notifyItemChanged(previousIndex)
        notifyItemChanged(currentPlayingIndex)
    }
}
