package kr.co.vividnext.sodalive.audition.applicant

import android.content.Context
import android.content.Intent
import android.media.AudioAttributes
import android.media.MediaPlayer
import android.net.Uri
import android.widget.Toast
import com.orhanobut.logger.Logger
import kr.co.vividnext.sodalive.audio_content.AudioContentPlayService
import kr.co.vividnext.sodalive.audio_content.player.AudioContentPlayerService
import java.io.IOException

class AuditionApplicantMediaPlayerManager(
    private val context: Context,
    private val updateUI: (Int, Boolean) -> Unit
) {
    private var mediaPlayer: MediaPlayer? = null
    private var currentPlayingApplicantId: Long = -1
    private var currentPlayingPosition: Int = -1

    fun pauseContent() {
        mediaPlayer?.pause()
        updateUI(currentPlayingPosition, false)
    }

    private fun resumeContent() {
        pauseAudioContentService()
        mediaPlayer?.start()
        updateUI(currentPlayingPosition, true)
    }

    fun stopContent() {
        mediaPlayer?.let {
            it.stop()
            it.release()
            mediaPlayer = null
        }
        currentPlayingApplicantId = -1
        currentPlayingPosition = -1
        updateUI(currentPlayingPosition, false)
    }

    fun toggleContent(position: Int, applicantId: Long, voiceUrl: String) {
        if (currentPlayingApplicantId == applicantId && currentPlayingPosition == position) {
            if (mediaPlayer?.isPlaying == true) {
                pauseContent()
            } else {
                resumeContent()
            }
        } else {
            playContent(position, applicantId, voiceUrl)
        }
    }

    private fun playContent(position: Int, applicantId: Long, voiceUrl: String) {
        pauseAudioContentService()
        stopContent()

        currentPlayingPosition = position
        currentPlayingApplicantId = applicantId

        mediaPlayer = MediaPlayer().apply {
            setAudioAttributes(
                AudioAttributes.Builder()
                    .setContentType(AudioAttributes.CONTENT_TYPE_MUSIC)
                    .setUsage(AudioAttributes.USAGE_MEDIA)
                    .build()
            )

            try {
                setDataSource(context, Uri.parse(voiceUrl))
                prepareAsync() // 비동기적으로 준비
                setOnPreparedListener {
                    start()
                    updateUI(currentPlayingPosition, true)
                }
            } catch (e: IOException) {
                e.printStackTrace()
                Toast.makeText(context, "콘텐츠를 재생하지 못했습니다.\n다시 시도해 주세요", Toast.LENGTH_SHORT).show()
            }

            setOnCompletionListener {
                updateUI(currentPlayingPosition, false)
                currentPlayingApplicantId = -1
            }
        }
    }

    private fun pauseAudioContentService() {
        context.startService(
            Intent(
                context,
                AudioContentPlayService::class.java
            ).apply {
                action = AudioContentPlayService.MusicAction.PAUSE.name
            }
        )

        context.startService(
            Intent(context, AudioContentPlayerService::class.java).apply {
                action = "STOP_SERVICE"
            }
        )
    }
}
